@@ -38,7 +38,7 @@ geom_label_repel <- function(
     if (!missing(position)) {
       stop("Specify either `position` or `nudge_x`/`nudge_y`", call. = FALSE)
     }
-    position <- position_nudge2(nudge_x, nudge_y)
+    position <- position_nudge_repel(nudge_x, nudge_y)
   }
   layer(
     data = data,
@@ -76,10 +76,11 @@ geom_label_repel <- function(
 }
 
 #' GeomLabelRepel
-#' @rdname ggplot2-ggproto
+#' @rdname ggrepel
 #' @format NULL
 #' @usage NULL
 #' @seealso \link[ggplot2]{GeomLabel} from the ggplot2 package.
+#' @keywords internal
 #' @export
 GeomLabelRepel <- ggproto(
   "GeomLabelRepel", Geom,
@@ -92,11 +93,11 @@ GeomLabelRepel <- ggproto(
     segment.linetype = 1, segment.colour = NULL, segment.size = 0.5, segment.alpha = NULL,
     segment.curvature = 0, segment.angle = 90, segment.ncp = 1,
     segment.shape = 0.5, segment.square = TRUE, segment.squareShape = 1,
-    segment.inflect = FALSE, segment.debug = FALSE,
+    segment.inflect = FALSE, segment.debug = FALSE
   ),
 
   draw_panel = function(
-    self, data, panel_scales, coord,
+    data, panel_scales, coord,
     parse = FALSE,
     na.rm = FALSE,
     box.padding = 0.25,
@@ -108,13 +109,13 @@ GeomLabelRepel <- ggproto(
     arrow = NULL,
     force = 1,
     force_pull = 1,
+    max.time = 0.5,
+    max.iter = 10000,
+    max.overlaps = 10,
     nudge_x = 0,
     nudge_y = 0,
     xlim = c(NA, NA),
     ylim = c(NA, NA),
-    max.time = 0.5,
-    max.iter = 10000,
-    max.overlaps = 10,
     direction = "both",
     seed = NA,
     verbose = FALSE
@@ -127,13 +128,19 @@ GeomLabelRepel <- ggproto(
       return()
     }
 
-    # position_nudge2() should have added these columns.
+    # if needed rename columns using our convention
     for (this_dim in c("x", "y")) {
+      this_orig <- sprintf("%s_orig", this_dim)
       this_nudge <- sprintf("nudge_%s", this_dim)
       if (!this_nudge %in% colnames(data)) {
         data[[this_nudge]] <- data[[this_dim]]
+        if (this_orig %in% colnames(data)) {
+          data[[this_dim]] <- data[[this_orig]]
+          data[[this_orig]] <- NULL
+        }
       }
     }
+
     # Transform the nudges to the panel scales.
     nudges <- data.frame(x = data$nudge_x, y = data$nudge_y)
     nudges <- coord$transform(nudges, panel_scales)
@@ -209,8 +216,6 @@ makeContent.labelrepeltree <- function(x) {
   if (is.na(x$point.padding)) {
     x$point.padding = unit(0, "lines")
   }
-  # point_padding_x <- convertWidth(x$point.padding, "native", valueOnly = TRUE)
-  # point_padding_y <- convertHeight(x$point.padding, "native", valueOnly = TRUE)
 
   # Do not create text labels for empty strings.
   valid_strings <- which(not_empty(x$lab))
@@ -219,8 +224,8 @@ makeContent.labelrepeltree <- function(x) {
   x$data <- x$data[ix,]
   x$lab <- x$lab[ix]
 
-  # Create a dataframe with x y width height
-  boxes <- lapply(valid_strings, function(i) {
+  # Create a dataframe with x1 y1 x2 y2
+  boxes <- lapply(seq_along(valid_strings), function(i) {
     row <- x$data[i, , drop = FALSE]
     t <- textGrob(
       x$lab[i],
@@ -380,13 +385,13 @@ makeLabelRepelGrobs <- function(
   label,
   x = unit(0.5, "npc"),
   y = unit(0.5, "npc"),
-  x.orig = unit(0.5, "npc"),
-  y.orig = unit(0.5, "npc"),
+  # Position of original data points.
+  x.orig = 0.5,
+  y.orig = 0.5,
   # Width and height of text boxes.
   box.width = 0,
   box.height = 0,
   default.units = "npc",
-  just = "center",
   box.padding = 0.25,
   label.padding = 0.25,
   point.size = 1,
